<?php

namespace FiloBlu\Storelocator\Block\Adminhtml\Stores\Edit\Tab;

class Tag extends \Magento\Backend\Block\Widget\Grid\Extended implements \Magento\Backend\Block\Widget\Tab\TabInterface
{
    /**
     * Tag collection factory
     *
     * @var \FiloBlu\Storelocator\Model\ResourceModel\Tags\CollectionFactory
     */
    protected $tagCollectionFactory;

    /**
     * Registry
     *
     * @var \Magento\Framework\Registry
     */
    protected $coreRegistry;

    /**
     * Tag factory
     *
     * @var \FiloBlu\Storelocator\Model\TagFactory
     */
    protected $tagFactory;

    /**
     * constructor
     *
     * @param \FiloBlu\Storelocator\Model\ResourceModel\Tags\CollectionFactory $tagCollectionFactory
     * @param \Magento\Framework\Registry $coreRegistry
     * @param \FiloBlu\Storelocator\Model\TagFactory $tagFactory
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Backend\Helper\Data $backendHelper
     * @param array $data
     */
    public function __construct(
        \FiloBlu\Storelocator\Model\ResourceModel\Tags\CollectionFactory $tagCollectionFactory,
        \Magento\Framework\Registry $coreRegistry,
        \FiloBlu\Storelocator\Model\TagsFactory $tagFactory,
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Backend\Helper\Data $backendHelper,
        array $data = []
    )
    {
        $this->tagCollectionFactory = $tagCollectionFactory;
        $this->coreRegistry         = $coreRegistry;
        $this->tagFactory           = $tagFactory;
        parent::__construct($context, $backendHelper, $data);
    }


    /**
     * Set grid params
     */
    public function _construct()
    {
        parent::_construct();
        $this->setId('tag_grid');
        $this->setDefaultSort('name');
        $this->setDefaultDir('ASC');
        $this->setUseAjax(true);
        if ($this->getStore()->getStoreId()) {
            $this->setDefaultFilter(['in_tags'=>1]);
        }
    }

    /**
     * prepare the collection

     * @return $this
     */
    protected function _prepareCollection()
    {
        /** @var \FiloBlu\Storelocator\Model\ResourceModel\Tags\Collection $collection */
        $collection = $this->tagCollectionFactory->create();

        if ($this->getStore()->getStoreId()) {
            $constraint = 'related.store_id='.$this->getStore()->getStoreId();
        } else {
            $constraint = 'related.store_id=0';
        }
        $collection->getSelect()->joinLeft(
            array('related' => $collection->getTable('filoblu_storelocator_store_tag')),
            'related.tag_id=main_table.tag_id AND '.$constraint,
            array('store_id')
        );

        $this->setCollection($collection);
        parent::_prepareCollection();
        return $this;
    }

    /**
     * @return $this
     */
    protected function _prepareMassaction()
    {
        return $this;
    }

    /**
     * @return $this
     */
    protected function _prepareColumns()
    {
        $this->addColumn(
            'in_tags',
            [
                'header_css_class'  => 'a-center',
                'type'   => 'checkbox',
                'name'   => 'in_tag',
                'values' => $this->_getSelectedTags(),
                'align'  => 'center',
                'index'  => 'tag_id'
            ]
        );
        $this->addColumn(
            'tag_id',
            [
                'header' => __('ID'),
                'sortable' => true,
                'index' => 'tag_id',
                'type' => 'number',
                'header_css_class' => 'col-id',
                'column_css_class' => 'col-id'
            ]
        );

        $this->addColumn(
            'tag_name',
            [
                'header' => __('Name'),
                'index' => 'name',
                'header_css_class' => 'col-name',
                'column_css_class' => 'col-name'
            ]
        );

        $this->addColumn(
            'tag_image',
            array(
                'header' => __('Image'),
                'index' => 'image',
                'renderer'  => '\FiloBlu\Storelocator\Block\Adminhtml\Stores\Grid\Renderer\Tag\Image',
            )
        );

        return $this;
    }

    /**
     * Retrieve selected Tags

     * @return array
     */
    protected function _getSelectedTags()
    {

        $tags = $this->getStoreTags();

        if (!is_array($tags)) {
            $tags = $this->getStore()->getOldTags();
            return $tags;
        }

        return $tags;
    }

    /**
     * Retrieve selected Tags

     * @return array
     */
    public function getSelectedTags()
    {

        $selected = $this->getStore()->getOldTags();

        return $selected;
    }

    /**
     * @param \FiloBlu\Storelocator\Model\Stores|\Magento\Framework\Object $item
     * @return string
     */
    public function getRowUrl($item)
    {
        return '#';
    }

    /**
     * get grid url
     *
     * @return string
     */
    public function getGridUrl()
    {
        return $this->getUrl(
            '*/*/tagsGrid',
            [
                'store_id' => $this->getStore()->getStoreId()
            ]
        );
    }

    /**
     * @return \FiloBlu\Storelocator\Model\Stores
     */
    public function getStore()
    {
        return $this->coreRegistry->registry('storelocator_store');
    }

    /**
     * @param \Magento\Backend\Block\Widget\Grid\Column $column
     * @return $this
     */
    protected function _addColumnFilterToCollection($column)
    {
        if ($column->getId() == 'in_tags') {
            $tagIds = $this->_getSelectedTags();

            if (empty($tagIds)) {
                $tagIds = 0;
            }

            if ($column->getFilter()->getValue()) {
                $this->getCollection()->addFieldToFilter('main_table.tag_id', ['in'=>$tagIds]);
                $this->getCollection()->addFieldToFilter('main_table.tag_id', ['in'=>$tagIds]);
            } else {
                if ($tagIds) {
                    $this->getCollection()->addFieldToFilter('main_table.tag_id', ['nin'=>$tagIds]);
                }
            }
        } else {
            parent::_addColumnFilterToCollection($column);
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getTabLabel()
    {
        return __('Tags');
    }

    /**
     * @return bool
     */
    public function isHidden()
    {
        return false;
    }

    /**
     * @return string
     */
    public function getTabTitle()
    {
        return $this->getTabLabel();
    }

    /**
     * @return bool
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * @return string
     */
    public function getTabUrl()
    {
        return $this->getUrl('filoblustorelocatoradmin/stores/tags', ['_current' => true]);
    }

    /**
     * @return string
     */
    public function getTabClass()
    {
        return 'ajax only';
    }
}
